import matplotlib.pyplot as plt
import numpy as np

import astropy.units as u
from astropy.io import fits

import sunpy.map
from sunpy.coordinates import sun
from sunpy.net import Fido
from sunpy.net import attrs as a

f = fits.open('20210427T080054_GBand.fits')
data = f[0].data
header = f[0].header

# Format the date properly
header['date-obs'] = f"{header['date'][0:4]}-{header['date'][4:6]}-{header['date'][6:11]}:{header['date'][11:13]}:{header['date'][13:16]}"

# Set the coordinate system to heliprojective
header['ctype1'] = 'HPLN-TAN'
header['ctype2'] = 'HPLT-TAN'
header['cunit1'] = 'deg'
header['cunit2'] = 'deg'

# Estimate the plate scale by eye
header['cdelt1'] = 0.00031
header['cdelt2'] = 0.00031

# Estimate the disk center by centroiding over-threshold pixels
row, col = np.nonzero(data > 1000)
header['crpix1'] = np.mean(col) + 1
header['crpix2'] = np.mean(row) + 1
print(f"CRPIX1: {header['crpix1']}, CRPIX2: {header['crpix2']}")

# Set the correct orientation
header['crota2'] = sun.P(header['date-obs']).to_value('deg') + 90

# Create the SunPy map
m = sunpy.map.Map(data, header)

# De-rotate the map
m = m.rotate()

# Download the HMI continuum image at the same time
result = Fido.search(a.Time(m.date, m.date + 1*u.min),
                     a.Instrument.hmi, a.Physobs.intensity)
print(result)
downloaded_file = Fido.fetch(result)

# Create the HMI map and de-rotate it
hmi_map = sunpy.map.Map(downloaded_file[0]).rotate()

# Plot the two maps side-by-sude with overlaid Stonyhurst heliographic grids
fig = plt.figure(figsize=(12, 5))

fig.add_subplot(121, projection=m)
m.plot()
m.draw_limb()
m.draw_grid(grid_spacing=10*u.deg)

fig.add_subplot(122, projection=hmi_map)
hmi_map.plot()
hmi_map.draw_limb()
hmi_map.draw_grid(grid_spacing=10*u.deg)

plt.show()
